#include <iostream>
#include <vector>
#include <set>
#include <cmath>
using namespace std;

#define TASK "player"

typedef long long ll;

struct Info {
    ll l, r, sm;
    int f, length;
    Info() {}
    Info(ll l, ll r, int f):l(l), r(r), f(f) {
        sm = 1LL * (r - l) * f;
        length = r - l;
    }

    bool operator < (const Info& other) const {
        return f > other.f || f == other.f && l < other.l;
    }

    bool operator == (const Info& other) const {
        return f == other.f && l == other.l && r == other.r;
    }

    int len() const {
        return length;
    }

    ll sum() const {
        return sm;
    }
};

void relax(const set<Info>& s, ll fun, ll sumT, const Info& last, double acel, double& ans) {
    const Info& end = *s.rbegin();
    double extraTime = fabs(fun) / end.f;
    if (end == last)
        ans = min(ans, 1.0 * (last.r - sumT) / acel + sumT - extraTime);
    else
        ans = min(ans, (last.r - sumT + extraTime) / acel + sumT - extraTime);
}

void update(set<Info>& s, ll& f, ll& sumT) {
    while (f + s.rbegin()->sum() <= 0) {
        f += s.rbegin()->sum();
        sumT -= s.rbegin()->len();
        s.erase(--s.end());
    }
}
int main() {
    freopen(TASK".in", "r", stdin);
    freopen(TASK".out", "w", stdout);

    int n, v, ff;
    scanf("%d%d%d", &n, &v, &ff);
    ll f = ff;

    vector<Info> a;
    ll sumT = 0;
    for (int i = 0; i < n; ++i) {
        int t, k;
        scanf("%d%d", &t, &k);
        for (int j = 0; j < k; ++j) {
            int l, r, fij;
            scanf("%d%d%d", &l, &r, &fij);
            a.push_back(Info(l + sumT, r + sumT, fij));
        }
        sumT += t;
    }

    set<Info> s;
    int i = 0;
    sumT = 0;
    for (; i < a.size() && f > 0; ++i) {
        f -= a[i].sum();
        sumT += a[i].len();
        s.insert(a[i]);
    }
    if (f > 0) {
        cout << -1 << endl;
        return 0;
    }

    double ans = 1e15;
    update(s, f, sumT);
    relax(s, f, sumT, a[i - 1], v, ans);

    for (; i < a.size(); ++i)
        if (s.rbegin()->f < a[i].f) {
            s.insert(a[i]);
            f -= a[i].sum();
            sumT += a[i].len();
            update(s, f, sumT);
            relax(s, f, sumT, a[i], v, ans);
        }
    printf("%.10f\n", ans);
    return 0;
}