import java.io.*;
import java.util.*;

public class TestGen {
    int testCount, subtaskCount;
    PrintWriter testsDesc;
    Random random = new Random(0x1_1_1_C_A);
    public static final int MAX_N = 100000;//10^5
    public static final int MAX_VAL = 1_000_000_000;

    private String folder;
    private String subtask;

    public void genSamples() throws IOException {
        setFolder("preliminary");
        setSubtask("subtask0");

        Test sam1 = new Test(3, 2, 5);
        sam1.newSong(4);
        sam1.addSegment(0, 1, 1);
        sam1.addSegment(2, 4, 1);
        sam1.newSong(6);
        sam1.addSegment(0, 1, 1);
        sam1.addSegment(1, 5, 4);
        sam1.newSong(3);
        sam1.addSegment(1, 3, 2);
        sam1.descr = "sample1";
        print(sam1);

        Test sam2 = new Test(2, 2, 10);
        sam2.newSong(3);
        sam2.addSegment(0, 1, 1);
        sam2.addSegment(1, 3, 1);
        sam2.newSong(2);
        sam2.addSegment(0, 2, 3);
        sam2.descr = "sample2";
        print(sam2);

        Test sam3 = new Test(4, 1, 8);
        sam3.newSong(5);
        sam3.addSegment(2, 4, 2);
        sam3.newSong(4);
        sam3.addSegment(1, 3, 1);
        sam3.newSong(3);
        sam3.addSegment(0, 1, 3);
        sam3.newSong(6);
        sam3.addSegment(0, 2, 10);
        sam3.addSegment(3, 5, 9);
        sam3.descr = "sample3";
        print(sam3);
        testCount = 0;
    }

    public void genHandleTests(int mx, int points) throws IOException {
    }

    @FunctionalInterface
    public interface SegmentsGenerator {
        int[] gen(Test test, int[] t, int sumK);
    }

    @FunctionalInterface
    public interface TiGenerator {
        int gen();
    }

    public class BoundGenerator implements TiGenerator {
        private int l, r;
        public BoundGenerator(int l, int r) {
            this.l = l;
            this.r = r;
        }

        @Override
        public int gen() {
            return random.nextInt(r - l + 1) + l;
        }
    }

    private SegmentsGenerator randomGenerator = (test, t, sumK)->{
        int[] segs = genRandCntSegs(sumK, test.n);
        for (int i = 0; i < segs.length; ++i)
            segs[i] = Math.min(t[i], segs[i]);
        return segs;
    };

    private SegmentsGenerator atEndGenerator = (test, t, sumK)->{
        int[] ret = new int[test.n];
        ret[test.n - 1] = Math.min(t[test.n - 1], sumK);
        return ret;
    };

    private TiGenerator simpleBoundGenerator = new BoundGenerator(1, MAX_VAL);

    private void genSegs(int n, int[] l, int[] r, int k) {
        Set<Integer> s = new HashSet<>();
        while (s.size() < k)
            s.add(random.nextInt(n));
        int sz = 0;
        for (Integer x : s)
            l[sz++] = x;
        Arrays.sort(l);
        for (int i = 0; i < k; ++i)
            r[i] = randInt((i == k - 1 ? n : l[i + 1]) - l[i]) + l[i];
    }

    private int[] genRandCntSegs(int n, int k) {
        int[] c = new int[k - 1];
        for (int i = 0; i < k - 1; ++i)
            c[i] = random.nextInt(n + 1);
        Arrays.sort(c);
        int[] ret = new int[k];
        ret[0] = c[0];
        for (int i = 1;  i < k - 1; ++i)
            ret[i] = c[i] - c[i - 1];
        ret[k - 1] = n - c[k - 2];
        return ret;
    }

    public void genSubtask(int n, int sumK, int maxV, int points) throws IOException {
        ++subtaskCount;
        String subtask = "subtask" + subtaskCount;
        setSubtask(subtask);
        setFolder("tests");

        BoundGenerator bg = new BoundGenerator(MAX_VAL / 10, MAX_VAL);
        print(genRandom(n, sumK, 1, 1, atEndGenerator, bg, MAX_VAL));
        print(genRandom(n, sumK, 1, 1, atEndGenerator, bg, 1));
        print(genRandom(n, sumK, 1, 2, atEndGenerator, bg, MAX_N / 2));

        print(genWrongSetSol(sumK, maxV));
        print(genWrongSetSol(sumK, maxV));
        print(genWrongSetSol(sumK, maxV));

        if (maxV > 1 && n == MAX_N) {
            print(genRandom(n, sumK, 2, 1, atEndGenerator, bg, MAX_VAL));
            print(genRandom(n, sumK, 2, 1, atEndGenerator, bg, 1));
            print(genRandom(n, sumK, 2, 2, atEndGenerator, bg, MAX_N));
        }

        print(genRandom(n, sumK, Math.min(maxV, randInt(MAX_VAL)), 100, randomGenerator, new BoundGenerator(1, 100), MAX_VAL));
        if (maxV != 1)
            print(genRandom(n, sumK, 1, 100, randomGenerator, new BoundGenerator(1, 10000), MAX_VAL));

        int pw = 1;
        for (int i = 0; i < 10; ++i) {
            print(genRandom(n, sumK, randInt(maxV), pw, randomGenerator));
            pw *= 10;
        }

        testsDesc.println(String.format("comment_subtask%d=\"n &lt;= %d, sumK &lt;= %d, v &lt;= %d\"", subtaskCount, n, sumK, maxV));
        testsDesc.println(String.format("bonus_subtask%d=%d", subtaskCount, points));
    }

    private Test genWrongSetSol(int sumK, int maxV) {
        int F = MAX_VAL;
        Test cur = new Test(1, randInt(Math.min(maxV, 10)), F);
        cur.newSong(MAX_VAL);
        int ans = 0;
        int s = 0;
        int c = randInt(50);
        int l = randInt(100);
        for (int i = 1; i < sumK; ++i) {
            int f = c;
            int len = l;
            cur.addSegment(s, s + len, f);
            ans += f;
            s += len + randInt(1000);
        }
        F -= ans;
        cur.addSegment(s, s + 2 * (F - ans) / c, c + 1);
        cur.descr = "anti gen wrong set solution";
        return cur;
    }

    private Test genRandom(int n, int sumK, int v, int maxFij, SegmentsGenerator generator) {
        return genRandom(n, sumK, v, maxFij, generator, simpleBoundGenerator, randInt(MAX_VAL));
    }

    private Test genRandom(int n, int sumK, int v, int maxFij, SegmentsGenerator generator, TiGenerator tiGenerator, int f) {
        int[] t = new int[n];
        for (int i = 0; i < n; ++i)
            t[i] = tiGenerator.gen();
        Test test = new Test(n, v, f);
        int[] cnt = generator.gen(test, t, sumK);

        for (int i = 0; i < n; ++i) {
            test.newSong(t[i]);
            int[] l = new int[cnt[i]];
            int[] r = new int[cnt[i]];
            genSegs(t[i], l, r, cnt[i]);
            for (int j = 0; j < cnt[i]; ++j)
                test.addSegment(l[j], r[j], randInt(maxFij));
        }

        int k = 0;
        for (int i = 0; i < n; ++i)
            k += test.left[i].size();

        test.descr = "random test n = " + n + " sumK = " + k  + " v = " + v + " maxFij = " + maxFij;
        return test;
    }


    private List<Integer> randomShuffle(int n) {
        List<Integer> p = new ArrayList<>();
        for (int i = 0; i < n; ++i) p.add(i);
        Collections.shuffle(p, random);
        return p;
    }

    private int randInt(int maxVal) {
        return random.nextInt(maxVal) + 1;
    }

    private long randLong(long maxVal) {
        return random.nextLong() % maxVal + 1;
    }

    public void setFolder(String folder) {
        this.folder = folder;
    }

    public void setSubtask(String subtask) {
        this.subtask = subtask;
    }

    public static class Test {
        public int n;
        public int v;
        public int F;
        public String descr;
        public ArrayList<Integer>[] left, right, fij;
        public int[] t;

        private int songNum = -1;
        public Test(int n, int v, int F) {
            this.n = n;
            this.v = v;
            this.F = F;
            t = new int[n];
            left = new ArrayList[n];
            right = new ArrayList[n];
            fij = new ArrayList[n];
            for (int i = 0; i < n; ++i) {
                left[i] = new ArrayList<>();
                right[i] = new ArrayList<>();
                fij[i] = new ArrayList<>();
            }
        }

        public void newSong(int tLen) {
            ++songNum;
            t[songNum] = tLen;
        }

        void addSegment(int l, int r, int fij) {
            this.left[songNum].add(l);
            this.right[songNum].add(r);
            this.fij[songNum].add(fij);
        }
    }

    public void print(Test cur) throws IOException {
        String path = String.format("../%s/%02d", folder, ++testCount);
        PrintWriter out = new PrintWriter(path);

        System.out.println("Generating test #" + testCount + ": " + cur.descr);

        out.println(String.format("%d %d %d", cur.n, cur.v, cur.F));
        for (int i = 0; i < cur.n; ++i) {
            out.print(String.format("%d %d", cur.t[i], cur.left[i].size()));
            for (int j = 0; j < cur.left[i].size(); ++j)
                out.print(String.format(" %d %d %d", cur.left[i].get(j), cur.right[i].get(j), cur.fij[i].get(j)));
            out.println();
        }

        out.close();
        testsDesc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=\"%s\"", path, folder, subtask, 0, cur.descr));
    }

    public void run() throws IOException {
        new File("../preliminary").mkdir();
        new File("../tests").mkdir();
        testsDesc = new PrintWriter("../tests.desc");

        genSamples();
        genSubtask(MAX_N, MAX_N, 1, 20);
        genSubtask(1000, 1000, MAX_VAL, 30);
        genSubtask(MAX_N, MAX_N, MAX_VAL, 50);

        testsDesc.close();
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
