import java.io.*;
import java.util.*;

public class TestGen {
    int testCount, subtaskCount;
    PrintWriter testsDesc;
    Random random = new Random(0x1_0_1_C_A);
    public static final int MAX_N = 100000;//10^5
    public static final int MAX_M = 100000;//10^5
    public static final int MAX_VAL = 10000;
    public static final int[] LOG_N = new int[MAX_N + 1];

    private String folder;
    private String subtask;

    public void genHandleTests(int mx, int points) throws IOException {
    }

    public void setFolder(String folder) {
        this.folder = folder;
    }

    public void setSubtask(String subtask) {
        this.subtask = subtask;
    }

    public void genSamples() throws IOException {
        setFolder("preliminary");
        setSubtask("subtask0");

        Test test1 = new Test(10, 5, 3, "1011110010", "0010010010");
        test1.addProc(0, 1);
        test1.addProc(1, 3);
        test1.addProc(3, 15);

        test1.addQuery(0, 9);
        test1.addQuery(1, 9);
        test1.addQuery(0, 0);
        test1.addQuery(1, 2);
        test1.addQuery(1, 3);
        print(test1);

        Test test2 = new Test(12, 6, 3, "101111111111", "010000000000");
        test2.addProc(0, 1);
        test2.addProc(1, 1);
        test2.addProc(3, 7);

        test2.addQuery(0, 10);
        test2.addQuery(1, 10);
        test2.addQuery(0, 0);
        test2.addQuery(1, 7);
        test2.addQuery(0, 9);
        test2.addQuery(1, 10);
        print(test2);
        testCount = 0;
    }

    @FunctionalInterface
    public interface QueryGenerator {
        Test gen(Test test, List<Integer> nums, List<Integer> pos);
    }

    private QueryGenerator straightOrder = (test, nums, pos)->{
        for (int i = 0; i < test.m / 2; ++i) {
            int id = random.nextInt(test.n);
            test.addQuery(nums.get(id), pos.get(id));
            test.addQuery(nums.get(id), pos.get(id));
        }
        return test;
    };

    private QueryGenerator shuffleOrder = (test, nums, pos)->{
        List<Integer> a = new ArrayList<>();
        List<Integer> b = new ArrayList<>();
        for (int i = 0; i < test.m / 2; ++i) {
            int id = random.nextInt(test.n);
            a.add(nums.get(id));
            a.add(nums.get(id));
            b.add(pos.get(id));
            b.add(pos.get(id));
        }
        synchronizeShuffle(a, b);
        for (int i = 0; i < a.size(); ++i)
            test.addQuery(a.get(i), b.get(i));
        return test;
    };

    private <T> void synchronizeShuffle(List<T>... lists) {
        int n = lists[0].size();
        List<Integer> p = randomShuffle(n);
        for (int i = 0; i < lists.length; ++i) {
            List<T> newlist = new ArrayList<>();
            for (Integer id : p)
                newlist.add(lists[i].get(id));
            lists[i] = newlist;
        }
    }

    private List<Integer> randomShuffle(int n) {
        List<Integer> p = new ArrayList<>();
        for (int i = 0; i < n; ++i) p.add(i);
        Collections.shuffle(p, random);
        return p;
    }

    public void genSubtask(int n, int m, int points) throws IOException {
        ++subtaskCount;
        String subtask = "subtask" + Integer.toString(subtaskCount);

        setSubtask(subtask);
        setFolder("tests");

        int pow2 = 1 << (LOG_N[n] - 1);
        print(allOnesStupid(n, m));
        print(allOnesStupid(n, m));
        print(allOnesStupid(pow2, m));
        print(allOnesStupid(pow2 - 1, m));

        print(allOnesRandQuery(n, m, 1, straightOrder));
        print(allOnesRandQuery(n, m, 1, straightOrder));
        print(allOnesRandQuery(pow2, m, 1, straightOrder));
        print(allOnesRandQuery(pow2 - 1, m, 1, straightOrder));

        print(allOnesRandQuery(n, m, LOG_N[n] / 2, straightOrder));
        print(allOnesRandQuery(n, m, LOG_N[n] / 2, straightOrder));
        print(allOnesRandQuery(pow2, m, LOG_N[n] / 2, straightOrder));
        print(allOnesRandQuery(pow2 - 1, m, LOG_N[n] - 1, straightOrder));

        print(allOnesRandQuery(n, m, LOG_N[n] - 3, shuffleOrder));
        print(allOnesRandQuery(n, m, LOG_N[n] - 4, shuffleOrder));
        print(allOnesRandQuery(pow2, m, LOG_N[n], shuffleOrder));

        double[] prob = new double[]{0.0, 0.001, 0.1, 0.2, 0.4, 0.6, 0.8, 0.9, 0.99, 0.999, 0.9999, 1.0};
        for (Double p : prob)
            print(genRandom(n, m, randInt(LOG_N[n] / 2), p));
        double[] prob2 = new double[]{0.0, 0.2, 0.4, 0.5, 0.6, 0.8, 1.0};
        for (Double p : prob2)
            print(genFullRandom(n, m, randInt(LOG_N[n] / 2), p));

        testsDesc.println(String.format("comment_subtask%d=\"n &lt;= %d, m &lt;= %d\"", subtaskCount, n, m));
        testsDesc.println(String.format("bonus_subtask%d=%d", subtaskCount, points));
    }

    private Test genFullRandom(int n, int m, int k, double probablyOne) {
        String a = randomString(n, probablyOne);
        String b = randomString(n, probablyOne);
        Test test = new Test(n, m, k, a, b);
        int[] p = randAny(k);
        int[] h = randDiffIncrease(k, LOG_N[n]);
        for (int i = 0; i < k; ++i)
            test.addProc(h[i], p[i]);

        for (int i = 0; i < n; ++i) {
            int num = random.nextInt(2);
            int id = random.nextInt(n);
            test.addQuery(num, id);
        }
        test.descr = "full random test with probably one = " + probablyOne + ", n = " + n + ", m = " + m + ", k = " + k;
        return test;
    }

    private Test genRandom(int n, int m, int k, double probablyOne) {
        StringBuilder sa = new StringBuilder(), sb = new StringBuilder();
        for (int i = 0; i < n; ++i)
            if (random.nextDouble() < probablyOne) {
                if (random.nextInt(2) == 0) {
                    sa.append('1');
                    sb.append('0');
                } else {
                    sa.append('0');
                    sb.append('1');
                }
            } else {
                sa.append('0');
                sb.append('0');
            }

        String a = sa.toString(), b = sb.toString();
        Test test = new Test(n, m, k, a, b);
        int[] p = randIncrease(k);
        int[] h = randDiffIncrease(k, LOG_N[n]);
        for (int i = 0; i < k; ++i)
            test.addProc(h[i], p[i]);

        for (int i = 0; i < n; ++i) {
            int num = random.nextInt(2);
            int id = random.nextInt(n);
            test.addQuery(num, id);
        }
        test.descr = "random test with probably one = " + probablyOne + ", n = " + n + ", m = " + m + ", k = " + k;
        return test;
    }

    private int[] randDiffIncrease(int k, int mxVal) {
        Set<Integer> diff = new TreeSet<>();
        while (diff.size() < k)
            diff.add(random.nextInt(mxVal));
        int[] ret = new int[k];
        int i = 0;
        for (Integer x : diff)
            ret[i++] = x;
        return ret;
    }

    private int[] randAny(int k) {
        int[] ret = new int[k];
        for (int i = 0; i < k; ++i)
            ret[i] = randInt(MAX_VAL);
        return ret;
    }

    private String randomString(int n, double probablyOne) {
        StringBuilder s = new StringBuilder();
        for (int i = 0; i < n; ++i)
            if (random.nextDouble() < probablyOne) s.append('1');
            else s.append('0');
        return s.toString();
    }

    void sumOneStrings(StringBuilder a, StringBuilder b, int n) {
        for (int i = 0; i < n; ++i)
            if (random.nextBoolean()) {
                a.append('1');
                b.append('0');
            } else {
                b.append('1');
                a.append('0');
            }

    }

    private Test allOnesRandQuery(int n, int m, int k, QueryGenerator gen) {
        StringBuilder a = new StringBuilder();
        StringBuilder b = new StringBuilder();
        sumOneStrings(a, b, n);

        Test test = new Test(n, m, k, a.toString(), b.toString());
        int[] p = randIncrease(k);
        for (int i = 0; i < k; ++i)
            test.addProc(i, p[i]);

        List<Integer> nums = new ArrayList<>();
        List<Integer> pos = new ArrayList<>();
        for (int i = 0; i < n; ++i)
            if (a.charAt(i) == '0') {
                nums.add(0);
                pos.add(i);
            } else {
                nums.add(1);
                pos.add(i);
            }
        test.descr = "all ones, random queries n = " +  n + ", m = " + m + ", k = " + k;
        return gen.gen(test, nums, pos);
    }

    private Test allOnesStupid(int n, int m) {
        StringBuilder a = new StringBuilder();
        StringBuilder b = new StringBuilder();
        sumOneStrings(a, b, n);

        Test test = new Test(n, m, 1, a.toString(), b.toString());
        test.addProc(0, randInt(MAX_VAL));
        int num;
        if (a.charAt(0) == '0') num = 0;
        else num = 1;

        for (int i = 0; i < m / 2; ++i) {
            test.addQuery(num, 0);
            test.addQuery(num, 0);
        }
        test.descr = "all ones stupid n = " +  n + ", m = " + m + ", k = " + 1;
        return test;
    }

    private int[] randIncrease(int k) {
        int[] ret = new int[k];
        for (int i = 0; i < k; ++i)
            ret[i] = randInt(MAX_VAL);
        Arrays.sort(ret);
        return ret;
    }

    private int randInt(int maxVal) {
        return random.nextInt(maxVal) + 1;
    }

    public static class Test {
        public int n;
        public int m;
        public int k;
        String descr;
        String a, b;
        List<Integer> h = new ArrayList<>();
        List<Integer> p = new ArrayList<>();
        List<Integer> num = new ArrayList<>();
        List<Integer> bit = new ArrayList<>();

        public Test(int n, int m, int k, String a, String b) {
            this.n = n;
            this.m = m;
            this.k = k;
            this.a = a;
            this.b = b;
        }

        public void addProc(int h, int p) {
            this.h.add(h);
            this.p.add(p);
        }

        public void addQuery(int num, int bit) {
            this.num.add(num);
            this.bit.add(bit);
        }
    }

    public void print(Test cur) throws IOException {
        String path = String.format("../%s/%02d", folder, ++testCount);
        PrintWriter out = new PrintWriter(path);

        int n = cur.n;
        int m = cur.m;
        int k = cur.k;
        cur.a = new StringBuilder(cur.a).reverse().toString();
        cur.b = new StringBuilder(cur.b).reverse().toString();
        System.out.println("Generating test #" + testCount + ": " + cur.descr);

        out.println(n + " " + k);
        out.println(cur.a);
        out.println(cur.b);
        for (int i = 0; i < k; ++i)
            out.println(cur.h.get(i) + " " + cur.p.get(i));
        out.println(m);
        for (int i = 0; i < m; ++i)
            out.println((cur.num.get(i) + 1) + " " + cur.bit.get(i));

        out.close();
        testsDesc.println(String.format("%s, folder=%s, subtask=%s, points=%d, comment=\"%s\"", path, folder, subtask, 0, cur.descr));
    }

    public void run() throws IOException {
        new File("../preliminary").mkdir();
        new File("../tests").mkdir();
        testsDesc = new PrintWriter("../tests.desc");
        genSamples();

        genSubtask(1000, 1000, 40);
        genSubtask(MAX_N, MAX_M, 60);

        testsDesc.close();
    }

    public static void main(String[] args) throws IOException {
        for (int i = 2; i <= MAX_N; ++i)
            LOG_N[i] = LOG_N[i / 2] + 1;
        for (int i = 1; i <= MAX_M; ++i)
            LOG_N[i]++;
        new TestGen().run();
    }
}
